@@ -121,7 +121,7 @@ phonemgr_utils_address_is (const char *addr)
 }
 
 static int
-get_rfcomm_channel (sdp_record_t *rec)
+get_rfcomm_channel (sdp_record_t *rec, gboolean only_gnapplet)
 {
 	int channel = -1;
 	sdp_list_t *protos = NULL;
@@ -135,6 +135,17 @@ get_rfcomm_channel (sdp_record_t *rec)
 	if (data)
 		name = g_strdup_printf ("%.*s", data->unitSize, data->val.str);
 
+	if (name == NULL)
+		goto end;
+
+	/* If we're only supposed to check for gnapplet, do it here
+	 * We ignore it if we're not supposed to check for it */
+	if (strcmp (name, "gnapplet") == 0) {
+		if (only_gnapplet != FALSE)
+			channel = sdp_get_proto_port (protos, RFCOMM_UUID);
+		goto end;
+	}
+
 	/* We can't seem to connect to the PC Suite channel */
 	if (strstr (name, "Nokia PC Suite") != NULL)
 		goto end;
@@ -158,7 +169,7 @@ end:
  * what the RFCOMM channel number for the service is.
  */
 static int
-find_service_channel (bdaddr_t *adapter, bdaddr_t *device, uint16_t svclass_id)
+find_service_channel (bdaddr_t *adapter, bdaddr_t *device, gboolean only_gnapplet, uint16_t svclass_id)
 {
 	sdp_session_t *sdp = NULL;
 	sdp_list_t *search = NULL, *attrs = NULL, *recs = NULL, *tmp;
@@ -170,17 +181,17 @@ find_service_channel (bdaddr_t *adapter, bdaddr_t *device, uint16_t svclass_id)
 	if (!sdp)
 		goto end;
 
-	sdp_uuid16_create(&browse_uuid, PUBLIC_BROWSE_GROUP);
 	sdp_uuid16_create(&service_id, svclass_id);
-	search = sdp_list_append (NULL, &browse_uuid);
-	search = sdp_list_append (search, &service_id);
+	attrs = sdp_list_append(0, &range);
+	search = sdp_list_append(0, &service_id);
 
-	attrs = sdp_list_append (NULL, &range);
+	g_message ("starting search");
 
 	if (sdp_service_search_attr_req (sdp, search,
 					 SDP_ATTR_REQ_RANGE, attrs,
-					 &recs))
+					 &recs)) {
 		goto end;
+	}
 
 	for (tmp = recs; tmp != NULL; tmp = tmp->next) {
 		sdp_record_t *rec = tmp->data;
@@ -188,7 +199,7 @@ find_service_channel (bdaddr_t *adapter, bdaddr_t *device, uint16_t svclass_id)
 		/* If this service is better than what we've
 		 * previously seen, try and get the channel number.
 		 */
-		channel = get_rfcomm_channel (rec);
+		channel = get_rfcomm_channel (rec, only_gnapplet);
 		if (channel > 0)
 			goto end;
 	}
@@ -203,7 +214,7 @@ end:
 }
 
 int
-phonemgr_utils_get_channel (const char *device)
+phonemgr_utils_get_serial_channel (const char *device)
 {
 	bdaddr_t src, dst;
 	int channel;
@@ -215,9 +226,31 @@ phonemgr_utils_get_channel (const char *device)
 	bacpy (&src, BDADDR_ANY);
 	str2ba(device, &dst);
 
-	channel = find_service_channel (&src, &dst, SERIAL_PORT_SVCLASS_ID);
+	channel = find_service_channel (&src, &dst, FALSE, SERIAL_PORT_SVCLASS_ID);
 	if (channel < 0)
-		channel = find_service_channel (&src, &dst, DIALUP_NET_SVCLASS_ID);
+		channel = find_service_channel (&src, &dst, FALSE, DIALUP_NET_SVCLASS_ID);
+
+	g_message ("Using serial channel %d for device %s", channel, device);
+
+	return channel;
+}
+
+int
+phonemgr_utils_get_gnapplet_channel (const char *device)
+{
+	bdaddr_t src, dst;
+	int channel;
+
+	/* If it's not a Bluetooth address */
+	if (bachk (device) < 0)
+		return -1;
+
+	bacpy (&src, BDADDR_ANY);
+	str2ba(device, &dst);
+
+	channel = find_service_channel (&src, &dst, TRUE, SERIAL_PORT_SVCLASS_ID);
+
+	g_message ("Using gnapplet channel %d for %s", channel, device);
 
 	return channel;
 }
@@ -277,19 +310,16 @@ phonemgr_utils_driver_for_model (const char *model, const char *device)
 
 	driver = g_hash_table_lookup (driver_model, model);
 	if (driver == NULL) {
-		g_message ("Model %s not supported natively", model);
+		g_message ("Model %s using default driver", model);
 		driver = g_strdup (PHONEMGR_DEFAULT_DRIVER);
 	} else {
 		driver = g_strdup (driver);
+		/* Add it to the list if it's a bluetooth device */
+		//FIXME this should also go in GConf
+		if (phonemgr_utils_address_is (device) == PHONEMGR_CONNECTION_BLUETOOTH)
+			g_hash_table_insert (driver_device, g_strdup (device), g_strdup (driver));
 	}
 
-	/* Add it to the list if it's a bluetooth device */
-	//FIXME this should also go in GConf
-	if (phonemgr_utils_address_is (device) == PHONEMGR_CONNECTION_BLUETOOTH) {
-		g_hash_table_insert (driver_device,
-				g_strdup (device),
-				driver);
-	}
 
 	return driver;
 }
@@ -407,8 +437,8 @@ phonemgr_utils_init_hash_tables (void)
 	if (driver_device != NULL && driver_model != NULL)
 		return;
 
-	driver_device = g_hash_table_new (g_str_hash, g_str_equal);
-	driver_model = g_hash_table_new (g_str_hash, g_str_equal);
+	driver_device = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	driver_model = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 	phonemgr_utils_start_parse ();
 }
@@ -521,7 +551,8 @@ phonemgr_utils_disconnect (PhonemgrState *phone_state)
 void
 phonemgr_utils_free (PhonemgrState *phone_state)
 {
-	g_return_if_fail (phone_state != NULL);
+	if (phone_state == NULL)
+		return;
 	g_free (phone_state);
 }
 
@@ -543,7 +574,7 @@ phonemgr_utils_tell_driver (const char *addr)
 	}
 
 	if (type == PHONEMGR_CONNECTION_BLUETOOTH) {
-		channel = phonemgr_utils_get_channel (addr);
+		channel = phonemgr_utils_get_serial_channel (addr);
 		if (channel < 0) {
 			g_warning ("Couldn't find the channel to connect to on Bluetooth device");
 			return;
@@ -590,7 +621,7 @@ phonemgr_utils_write_gnokii_config (const char *addr)
 	}
 
 	if (type == PHONEMGR_CONNECTION_BLUETOOTH) {
-		channel = phonemgr_utils_get_channel (addr);
+		channel = phonemgr_utils_get_serial_channel (addr);
 		if (channel < 0) {
 			g_warning ("Couldn't find the channel to connect to on Bluetooth device");
 			return;
